#!/usr/bin/env python
import pandas as pd
from toxassign import Format
import os
import logging


# collate unfound files together
def unfound_merge():
    merge_main("UnfoundCopy", "Unfound")


# collate unchecked files together
def unchecked_merge():
    print("---- UNCHECKED MERGE ----")
    merge_main("Unchecked", "Unchecked")


# collate toxic files together
def toxic_merge():
    print("---- TOXIC MERGE ----")
    checked = pd.DataFrame(columns=['name', 'toxicity'])

    for subDir in os.scandir("."):
        # track whether directory has changed
        chdir = False
        # merge files into one object
        try:
            os.chdir(f"./{subDir.name}")
            chdir = True
            # read in toxic file
            unfound = pd.read_csv(f"./+ SetToxic.txt", delimiter="\t",
                                  engine='python', names=["name", "toxicity"])
            # merge unfound into checked
            checked = unfound.merge(checked, how="outer", on="name")
            # read in toxic file
            unfound = pd.read_csv(f"./+ SetToxic.txt", delimiter="\t",
                                  engine='python', names=["name", "toxicity"])
            # repeat
            checked = unfound.merge(checked, how="outer", on="name")
            os.chdir("../src")
            chdir = False
        except Exception as e:
            logging.exception(e)
            if chdir:
                os.chdir("../src")

    # write new collated object to file

    with open(f"./totalTox.txt", "w+") as fTotalTox:
        for index, value in checked.iterrows():
            fTotalTox.write(str(value["name"]) + "\t" + str(value["toxicity"]) + "\n")
    # format new collated file
    Format.toxfilter(".", "totalTox")


def merge_main(infile, outfile):
    checked = pd.DataFrame(columns=['Compound'])

    for subdir in os.scandir("."):  # merge files into one object
        try:
            os.chdir(f"./{subdir.name}")  # move into data directory
            # read in toxic file
            unfound = pd.read_csv(f"./+ Set{infile}.txt", delimiter="\t", engine='python', names=["Compound"])
            checked = unfound.merge(checked, how="outer")  # merge unfound into checked
            unfound = pd.read_csv(f"./- Set{infile}.txt", delimiter="\t", engine='python', names=["Compound"])  # repeat
            checked = unfound.merge(checked, how="outer")
        finally:
            # move out of dir at end
            os.chdir("..")

    # write object to file
    print(os.getcwd())
    with open(f"./total{outfile}.txt", "w+") as fTotalUnfound:
        for index, value in checked.iterrows():
            fTotalUnfound.write(str(value["Compound"]) + "\n")



#!/usr/bin/env python
from requests.exceptions import Timeout
import requests as req
import pandas as pd
import json
import time
import logging


class PubChemClass:
    toxic = set()
    safe = set()
    unknown = set()
    unchecked = set()
    timeout = set()
    unfound = set()

    def foodrecordcheck(record, safe, index):
        try:
            for j in range(5):
                # Chemical is generally recognised as safe
                if record["Record"]["Section"][index]["Section"][j]["TOCHeading"] == \
                        "FDA Generally Recognized as Safe - GRAS Notices":
                    # print("Found! FDA GRAS!")
                    safe.add(str(record["Record"]["RecordTitle"]) + ", FDA GRAS")
                    return True
                # Chemical is added to food
                if record["Record"]["Section"][index]["Section"][j]["TOCHeading"] == "FDA Substances Added to Food":
                    # print("Found! FDA food usages!")
                    safe.add(record["Record"]["RecordTitle"] + ", FDA food use")
                    return True
                # chemical is added to food
                if record["Record"]["Section"][index]["Section"][j]["TOCHeading"] == "Food Additive Classes":
                    # print("Found! Food Additive!")
                    safe.add(record["Record"]["RecordTitle"] + ", Food Additive")
                    return True

        except IndexError:
            # print("food record not found")
            return True

    def toxicrecordcheck(record, toxic):
        # print("toxic record check placeholder")
        toxic.add(record["Record"]["RecordTitle"] + " nan")
        # todo no scanning for toxic records
        return

    def safetyhazardcheck(record, toxic, index):
        # print("safety hazard check |", end=" ")
        try:
            for j in range(5):
                if record["Record"]["Section"][index]["Section"][0]["Section"][j]["TOCHeading"] \
                        == "Hazard Classes and Categories":
                    # add hazard classes to file such as acute tox and irritability
                    # print("Found! Hazard classes added to file")
                    toxic.add(record["Record"]["RecordTitle"] + "\t" +
                              str(record["Record"]["Section"][index]["Section"][0]["Section"]
                                  [j]["Information"][0]["Value"]["StringWithMarkup"]))
                    return True
        except IndexError:
            # print("hazard class not found |", end="")
            return False

    def request(cids, unknown, unchecked, timeout, toxic, safe):
        ids = cids["IdentifierList"]["CID"]
        for cid in ids:
            try:
                response = req.get(
                    "https://pubchem.ncbi.nlm.nih.gov/rest/pug_view/data/compound/{cid}/JSON/".format(
                        cid=cid))  # request full record about chemical
                time.sleep(.2)  # wait to not overflow PubChemClass
                record = json.loads(response.text)
                checked = False  # whether record has been checked to not double count
                try:
                    for i in range(100):
                        if checked:
                            continue
                        if record["Record"]["Section"][i]["TOCHeading"] == "Toxicity":
                            PubChemClass.toxicrecordcheck(record, toxic)
                        elif record["Record"]["Section"][i]["TOCHeading"] == "Safety and Hazards":
                            checked = PubChemClass.safetyhazardcheck(record, toxic, i)
                except Exception as e:
                    # logging.exception(e)
                    pass
                for i in range(100):
                    if checked:
                        return
                    if record["Record"]["Section"][i]["TOCHeading"] == "Food Additives and Ingredients":
                        checked = PubChemClass.foodrecordcheck(record, safe, i)

            except Timeout:
                print()
                print("------TIMEOUT-------")
                print(cids)
                print("------------------")
                timeout.add(cid)
                return
            except IndexError:  # record not classifiable
                # print("not checked")
                unchecked.add(record["Record"]["RecordTitle"])
                return
            except Exception as e:
                logging.exception(e)
                unknown.add(cid)
                return

    def processchems(compounds, unknown, unchecked, timeout, unfound, toxic, safe):
        for value in compounds:
            print("[" + str(value) + "]")
            try:
                response = req.get(
                    "https://pubchem.ncbi.nlm.nih.gov/rest/pug/compound/name/{compound}/cids/JSON".format(
                        compound=value))
                time.sleep(.2)
                cids = json.loads(response.text)
                if response.status_code == 404:  # chemical not found in PubChemClass
                    unfound.add(value)
                    # print("404 not found")
                else:
                    PubChemClass.request(cids, unknown, unchecked, timeout, toxic, safe)

            except Timeout:
                print()
                print("------TIMEOUT-------")
                print(value)
                print("------------------")
                timeout.add(value[1])
                return
            except Exception as e:
                logging.exception(e)
                unknown.add(value[1])
                return

    def main(self, compound, sign):
        tox_names = set()
        esi = pd.read_csv(f'./{sign} {compound}.csv', encoding='unicode_escape')
        tox = pd.read_csv('./OpenFoodTox.csv', encoding='unicode_escape') \
            .dropna(axis=0, subset=['MOLECULARFORMULA', 'COM_NAME']).drop_duplicates()
        tox_names.update(tox.COM_NAME[tox.MOLECULARFORMULA.isin(esi.formula)])
        try:
            PubChemClass.processchems(tox_names, PubChemClass.unknown, PubChemClass.unchecked, PubChemClass.timeout,
                                      PubChemClass.unfound, PubChemClass.toxic, PubChemClass.safe)
        finally:
            print("done")
            # if not os.path.exists(f"./{compound}/"):  # create new directory for chem and move into it
            #     os.makedirs(f"./{compound}/")
            # os.chdir(f"./{compound}/")
            # write main out with unknown, timeout, and safe categories
            # with open(f"{sign} MainOut.txt", 'w+') as fOut:
            #     fOut.write("===================== Unknown =====================\n")
            #     for element in unknown:
            #         fOut.write(str(element))
            #         fOut.write("\n")
            #     fOut.write("===================== Timeout =====================\n")
            #     for element in timeout:
            #         fOut.write(str(element))
            #         fOut.write("\n")
            #     fOut.write("====================== Safe =======================\n")
            #     for element in safe:
            #         fOut.write(str(element))
            #         fOut.write("\n")
            # # write set unchecked with unchecked records
            # with open(f"{sign} SetUnchecked.txt", 'w+') as fUnchecked:
            #     for element in unchecked:
            #         fUnchecked.write(str(element))
            #         fUnchecked.write("\n")
            # # write set unchecked with unfound records
            # with open(f"{sign} SetUnfound.txt", 'w+') as fUnfound:
            #     for element in unfound:
            #         fUnfound.write(str(element))
            #         fUnfound.write("\n")
            # # write set unchecked with toxic records
            # with open(f"{sign} SetToxic.txt", 'w+') as fToxic:
            #     for element in toxic:
            #         fToxic.write(str(element).replace("\'", "\""))
            #         fToxic.write("\n")

        print(PubChemClass.timeout)
        print("\n")
        print(PubChemClass.unknown)
        print("\n")
        # os.chdir("../src")



from openpyxl import Workbook
from openpyxl.utils.dataframe import dataframe_to_rows
import pandas


def write_chem(cmpd, sign, format, match, pub):
    wb = Workbook()
    toxic = wb.create_sheet("toxic")
    wb.remove_sheet(wb.get_sheet_by_name('Sheet'))
    # unknown = wb.create_sheet("unknown")
    # safe = wb.create_sheet("safe")
    # found = wb.create_sheet("found")

    for r in format.set1:
        toxic.append([cmpd, sign, "tox 1", r])

    for r in format.set2:
        toxic.append([cmpd, sign, "tox 2", r])

    for r in format.set3:
        toxic.append([cmpd, sign, "tox 3", r])

    for r in format.set4:
        toxic.append([cmpd, sign, "tox 4", r])

    # for r in match.set_unfound:
    #     unknown.append([r])
    #
    # for r in pub.safe:
    #     safe.append([r])
    #
    # for r in match.found_comp:
    #     found.append([r])

    wb.save(cmpd + sign + ".xlsx")



#!/usr/bin/env python
from toxassign import PubChemClass
from toxassign import Format
from toxassign import Match
from toxassign import Merge
from toxassign import Writer
import os
import logging

__author__ = "Samuel Breuer"
__copyright__ = "Copyright 2021, MOST"
__credits__ = ["Samuel Breuer", "Simeon Schum", "Joshua Pearce", "Lucy Toppen"]
__license__ = "MIT"
__version__ = "0.2.0"
__maintainer__ = "Samuel Breuer"
__email__ = "swbreuer@mtu.edu"
__status__ = "Development"


def automate(filepath):
    pub = PubChemClass()
    form = Format(pub)
    mat = Match(pub)
    os.chdir(filepath)
    # run against all data files in folder
    for root, subDirs, files in os.walk("."):
        for file in files:
            pub = PubChemClass()
            format = Format()
            match = Match()
            sign = file[0]
            cmpd = file.replace(f"{sign}", "").replace(".csv", "")

            # all data files are .csv and contain either - or + as their first character, all other files are not data
            if (file[-1] != "v") | ((sign != "-") & (sign != "+")):
                continue

            # create storage folder
            if not os.path.exists(cmpd):
                os.mkdir(cmpd)
            # call the pubchem record, classify, sort toxic records, match unfound records

            try:
                print(f"{sign}{cmpd}")
                pub.main(cmpd, sign)
                form.toxfilter()
                mat.match()
                Writer.write_chem(cmpd, sign, form, mat, pub)

            except Exception as e:
                logging.exception(e)
                pass

    # collate all universal records into total files
    Merge.unfound_merge()
    Merge.unchecked_merge()
    Merge.toxic_merge()
    Format.toxfilter(".", "totalTox")


def main():
    automate(".")


if __name__ == "__main__":
    automate(".")



#!/usr/bin/env python
import pandas as pd
import os
from toxassign import PubChem


class Match:
    source = PubChem
    set_unfound = set()
    set_safe = set()
    found_comp = set()

    def __init__(self, pub=PubChem):
        self.source = pub

    def match(self):
        print("---- UNFOUND MATCH ----")
        self.unfound = pd.DataFrame(list(Match.source.PubChemClass.unfound), columns=["name"])
        found = pd.read_csv("Remove.csv")

        self.unfound.name = self.unfound.name.astype(str)
        found.compound = found.compound.astype(str)
        self.found_comp = self.unfound.merge(found, right_on="compound", left_on="name", how="inner")
        self.set_safe = self.found_comp
        self.set_safe = self.set_safe[(self.set_safe.Safety == "safe") | (self.set_safe.Safety == "flavoring agent")
                                        | (self.set_safe.Safety == "fragrance") | (
                                                    self.set_safe.Safety == "supplement")]
        self.found_comp = self.found_comp[~self.found_comp.compound.isin(self.set_safe.compound)]
        self.set_unfound.update(self.unfound[~self.unfound.name.isin(found.compound)])

        # with open(f"./{compound}/{sign} SetFound.txt", "w+") as fSetFound:
        #     for index, value in found_comp.iterrows():
        #         fSetFound.write(str(value["compound"]) + ",\t" + str(value["alias"]) + ",\t" + str(value["Safety"]) + "\n")
        #     fSetFound.write("\n========= SAFE =========\n\n")
        #     for index, value in set_safe.iterrows():
        #         fSetFound.write(str(value["compound"]) + ",\t" + str(value["alias"]) + ",\t" + "\n")
        #
        # with open(f"./{compound}/{sign} SetUnfoundCopy.txt", "w+") as fSetUnfound:
        #     for element in set_unfound:
        #         fSetUnfound.write(element + "\n")

        # os.remove(f"./{compound}/{sign} SetUnfound.txt")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['toxassign']

package_data = \
{'': ['*']}

install_requires = \
['openpyxl>=3.0.9,<4.0.0', 'pandas>=1.3.5,<2.0.0']

entry_points = \
{'console_scripts': ['toxassign = toxassign.Automation:main']}

setup_kwargs = {
    'name': 'toxassign',
    'version': '0.5.4',
    'description': 'A toxic assigner used to classify potentially toxic chemicals from a list of formulas of compounds.',
    'long_description': '# ToxAssign\n[![Scrutinizer Code Quality](https://scrutinizer-ci.com/g/swbreuer/ToxAssign/badges/quality-score.png?b=main)](https://scrutinizer-ci.com/g/swbreuer/ToxAssign/?branch=main)\n[![Build Status](https://scrutinizer-ci.com/g/swbreuer/ToxAssign/badges/build.png?b=main)](https://scrutinizer-ci.com/g/swbreuer/ToxAssign/build-status/main)\n[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)\n[![MOST logo](https://www.appropedia.org/w/images/thumb/c/c2/Sunhusky.png/240px-Sunhusky.png)](https://www.appropedia.org/Category:MOST)\n\n`ToxAssign` is designed to take the output of [MFAssignR](https://github.com/skschum/MFAssignR) and process the toxic compounds into an easily readable and understandable format.\n\nContents\n========\n\n * [Why?](#why)\n * [Installation](#installation)\n * [Usage](#usage)\n * [Output](#output)\n \n ### Why?\n \n&nbsp;&nbsp;&nbsp;This project has been designed in tandem with MOST\'s [BioPROTEIN project](https://www.appropedia.org/BioPROTEIN) to act as a precursor to live animal testing of completely novel food sources. This is intended to decrease the number of live animal tests required as to both decrease cost and increase ethical use of research resources. This project also seeks to serve as a useful tool in researching potential food sources for desperate times such as common agricultural wastes and potential future reprocessing research. To mee this goal this project uses entirely open source and free to use tools in its full workflow, including [MZMine](http://mzmine.github.io) and the previously mentioned [MFAssignR](https://github.com/skschum/MFAssignR).\n \n \n ### Installation\n The installation of this project is very simple.\n \n Install with [`pip3`](https://pypi.org/project/shallow-backup/)  \n &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;`$ pip3 install ToxAssign`\n\n### Usage\n&nbsp;&nbsp;&nbsp;To operate this tool, the user will need to place the csv files containing formulas they want to process in an empty folder along with the open food tox csv and the remove csv as in figure 29. The OpenFoodTox file contains the database of toxic compounds with their related formulas and the Remove file contains compounds that are not found on the PubChemClass database and thus are classified by hand.\n![Initial Folder Contents](./Images/InitialFolderContents.png?raw=true)\nThe formulas must match in format and must have a header above them labeled “formula” to be properly recognized. The files must also have the format of "(sign) (compound).csv" to be properly recognized. Then, the user will open a terminal window inside of the folder with the data to be processed and run the command toxassign. This will begin to output to the terminal with an output that looks similar to the following figure, starting with the name of the first compound to be processed followed by a print out of all the matched compounds and their assignment.\n![Output](./Images/output.png)\nFinally, there may be records that are not recognized by PubChemClass\'s database or are under a different name. To manage these a local database has been collated to deal with and categorize these compounds. When you classify unfound records, ensure to add them to the `Remove.csv` file for later use in the format already demonstrated in the file. The safety column falls into two broad categories: `safe`, determined by the keywords `safe`, `flavoring agent`, `fragrance`, or `supplement`; and as `other` determined by any other keywords.\n### Output\nOnce the code has terminated the directory will contain 4 new files and one directory per compound. The 4 new files, totalTox, totalToxFiltered, totalUnchecked, and totalUnfound, will contain all of the toxic records, all of the toxic records sorted by toxicity, all of the records not sorted, and all of the records not found in the PubChemClass database. \n![final folder contents](./Images/finalFolderContents.png)\nThe folders created by ToxAssign are each named after a compound delivered in the input. Each folder contains "+/- MainOut”, “+/- SetFound”, “+/- SetToxicFiltered”, “+/- SetUnchecked”, and “+/- SetUnfoundCopy”.\n![compound Folder Contents](./Images/compoundFolderContents.png)\nMainOut contains the compounds that either had a large unknown error, those that timed out when accessing the PubChemClass server, or those that were deemed safe by being food additives. \n![Main Out Contents](./Images/MainOutContents.png)\nSetFound contains compounds that were not found on PubChemClass but were identified in the list of compounds found by hand, sorted by unsafe and safe. \n![Set Found Contents](./Images/setFoundContents.png)\nSetUnchecked contains compounds that were found on PubChemClass but did not have enough information and SetUnfound contains compounds that were not found on PubChemClass or in the list of compounds found by hand. Finally SetToxic contains all the compounds that contain either safety classes or toxic records, at the top by acute toxicity and at the bottom containing their safety classes or no data for toxic records. \n![Set Toxic Contents](./Images/setToxicContents.png)\n',
    'author': 'Samuel Breuer',
    'author_email': 'swbreuer@mtu.edu',
    'maintainer': 'Samuel Breuer',
    'maintainer_email': 'swbreuer@mtu.edu',
    'url': 'https://github.com/swbreuer/ToxAssign',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python
import pandas as pd
import json
import logging
from toxassign import PubChem


class Format:
    set1 = set()
    set2 = set()
    set3 = set()
    set4 = set()
    dict_else = dict()
    source = PubChem

    def __init__(self, pub=PubChem):
        self.source = pub

    def toxfilter(self):
        print("---- TOXIC FILTER ----")

        toxin = list(self.source.toxic)
        toxic = list()
        for r in toxin:
            toxic.append(r.split("\t"))
        toxic = pd.DataFrame(toxic, columns=["name", "toxicity"])

        for index, value in toxic.iterrows():
            toxfound = 5  # only mark highest toxicity
            try:
                data = json.loads(value["toxicity"].replace("'", '"'))
                for i in range(20):
                    if data[i]["String"][0:12] == "Acute Tox. 1":
                        self.set1.add(value["name"])
                        toxfound = 1
                    elif data[i]["String"][0:12] == "Acute Tox. 2" and toxfound > 1:
                        self.set2.add(value["name"])
                        toxfound = 2
                    elif data[i]["String"][0:12] == "Acute Tox. 3" and toxfound > 2:
                        self.set3.add(value["name"])
                        toxfound = 3
                    elif data[i]["String"][0:12] == "Acute Tox. 4" and toxfound > 3:
                        self.set4.add(value["name"])
                        toxfound = 4
            except Exception as e:
                # logging.exception(e)
                if toxfound > 4:
                    self.dict_else[value["name"]] = value[
                        "toxicity"]  # if not an acute toxic compound dump it in "other"

        # f = open(f"./{compound}/{filename}Filtered.txt", "w+")
        #
        # f.write("    ---------Toxic 1---------\n")
        # for element in set1:
        #     f.write(str(element))
        #     f.write("\n")
        # f.write("    ---------Toxic 2---------\n")
        # for element in set2:
        #     f.write(str(element))
        #     f.write("\n")
        # f.write("    ---------Toxic 3---------\n")
        # for element in set3:
        #     f.write(str(element))
        #     f.write("\n")
        # f.write("    ---------Toxic 4---------\n")
        # for element in set4:
        #     f.write(str(element))
        #     f.write("\n")
        # f.write("    ---------Else---------\n")
        # for key in dict_else:
        #     f.write(key)
        #     f.write("\t")
        #     try:
        #         f.write(dict_else[key])
        #     except Exception as e:
        #         logging.exception(e)
        #         pass
        #     f.write("\n")
        #
        # f.close()
